@@ -16,14 +16,14 @@
 #ifndef LIBRADIUS_H
 #define LIBRADIUS_H
 /*
- * $Id: 018a018f9e5182495024c6ad8dc3121bd6aca335 $
+ * $Id: 4dda31ced645438d4c2de1a5d7f47f860dc220d3 $
  *
  * @file libradius.h
  * @brief Structures and prototypes for the radius library.
  *
  * @copyright 1999-2014 The FreeRADIUS server project
  */
-RCSIDH(libradius_h, "$Id: 018a018f9e5182495024c6ad8dc3121bd6aca335 $")
+RCSIDH(libradius_h, "$Id: 4dda31ced645438d4c2de1a5d7f47f860dc220d3 $")
 
 /*
  *  Compiler hinting macros.  Included here for 3rd party consumers
@@ -411,6 +411,10 @@ typedef struct radius_packet {
 	bool			radiusv11;
 #endif
 	bool			tls;		//!< uses secure transport
+
+	bool			message_authenticator;
+	bool			proxy_state;
+	bool			eap_message;
 } RADIUS_PACKET;
 
 typedef enum {
@@ -532,7 +536,8 @@ bool		rad_packet_ok(RADIUS_PACKET *packet, int flags, decode_fail_t *reason);
 /*
  *	1 == require_ma
  *	2 == msg_peek
- *	3 == limit_proxy_state
+ *	4 == limit_proxy_state
+ *	8 == require_ma for Access-* replies and Protocol-Error
  */
 RADIUS_PACKET	*rad_recv(TALLOC_CTX *ctx, int fd, int flags);
 ssize_t rad_recv_header(int sockfd, fr_ipaddr_t *src_ipaddr, uint16_t *src_port, int *code);
@@ -727,7 +732,7 @@ extern bool	fr_dns_lookups;	/* do IP -> hostname lookups? */
 extern bool	fr_hostname_lookups; /* do hostname -> IP lookups? */
 extern int	fr_debug_lvl;	/* 0 = no debugging information */
 extern uint32_t	fr_max_attributes; /* per incoming packet */
-#define	FR_MAX_PACKET_CODE (52)
+#define	FR_MAX_PACKET_CODE (53)
 extern char const *fr_packet_codes[FR_MAX_PACKET_CODE];
 #define is_radius_code(_x) ((_x > 0) && (_x < FR_MAX_PACKET_CODE))
 extern FILE	*fr_log_fp;
@@ -922,6 +927,7 @@ rbnode_t	*rbtree_find(rbtree_t *tree, void const *data);
 void		*rbtree_finddata(rbtree_t *tree, void const *data);
 uint32_t	rbtree_num_elements(rbtree_t *tree);
 void		*rbtree_node2data(rbtree_t *tree, rbnode_t *node);
+void		*rbtree_first(rbtree_t *tree);
 
 /*
  *	The callback should be declared as:
@@ -965,6 +971,12 @@ int		fr_socket_wait_for_connect(int sockfd, struct timeval *timeout);
 }
 #endif
 
+typedef enum {
+	FR_BOOL_FALSE = 0,
+	FR_BOOL_TRUE,
+	FR_BOOL_AUTO,
+} fr_bool_auto_t;
+
 #include <freeradius-devel/packet.h>
 
 #ifdef WITH_TCP
