@@ -16,7 +16,7 @@
 #ifndef LISTEN_H
 #define LISTEN_H
 /**
- * $Id: e8222a3f025c756e0be1b4e6038ad4fd43843f2d $
+ * $Id: 15d00d1ba1a80c2fa33b5d2b5052d7008cc793d2 $
  *
  * @file listen.h
  * @brief The listener API.
@@ -76,14 +76,33 @@ struct rad_listen {
 	rad_listen_t	*parent;
 
 	bool		dual;
+	bool		proxy_protocol;		//!< haproxy protocol
 #endif
 	bool		nodup;
 	bool		synchronous;
+	bool		dead;
 	uint32_t	workers;
 
 #ifdef WITH_TLS
 	fr_tls_server_conf_t *tls;
 	bool		check_client_connections;
+	bool		nonblock;
+	bool		blocked;
+#ifdef WITH_RADIUSV11
+	fr_radiusv11_t 	radiusv11;
+#endif
+
+#ifdef WITH_COA_TUNNEL
+	char const	*key;		/* Originating-Realm-Key */
+	bool		send_coa;	/* to the NAS */
+
+	uint32_t	coa_irt;
+	uint32_t	coa_mrc;
+	uint32_t	coa_mrt;
+	uint32_t	coa_mrd;
+
+	int		num_ids_used;	/* for proxying CoA packets */
+#endif
 #endif
 
 	rad_listen_recv_t recv;
@@ -153,21 +172,34 @@ typedef struct listen_socket_t {
 	RADCLIENT	*client;
 
 	RADIUS_PACKET   *packet; /* for reading partial packets */
+
+	fr_ipaddr_t	haproxy_src_ipaddr;	//!< for proxy_protocol
+	fr_ipaddr_t	haproxy_dst_ipaddr;
+	uint16_t	haproxy_src_port;
+	uint16_t	haproxy_dst_port;
 #endif
 
 #ifdef WITH_TLS
 	tls_session_t	*ssn;
 	REQUEST		*request; /* horrible hacks */
 	VALUE_PAIR	*certs;
+	uint32_t	connect_timeout;
 	pthread_mutex_t mutex;
 	uint8_t		*data;
 	size_t		partial;
+
+	fr_event_fd_handler_t write_handler;
 	enum {
 		LISTEN_TLS_INIT = 0,
 		LISTEN_TLS_CHECKING,
 		LISTEN_TLS_SETUP,
 		LISTEN_TLS_RUNNING,
 	} state;
+
+#ifdef WITH_RADIUSV11
+	bool		alpn_checked;
+	bool		radiusv11;		//!< defaults to "no"!
+#endif
 #endif
 
 	RADCLIENT_LIST	*clients;
