@@ -16,7 +16,7 @@
 #ifndef RADIUSD_H
 #define RADIUSD_H
 /**
- * $Id: 9cfa418f228aeeb60f2c52b433211c38ceadfaea $
+ * $Id: ec69b8d4e096bef619e3541bb500744a6644abbd $
  *
  * @file radiusd.h
  * @brief Structures, prototypes and global variables for the FreeRADIUS server.
@@ -24,7 +24,7 @@
  * @copyright 1999-2000,2002-2008  The FreeRADIUS server project
  */
 
-RCSIDH(radiusd_h, "$Id: 9cfa418f228aeeb60f2c52b433211c38ceadfaea $")
+RCSIDH(radiusd_h, "$Id: ec69b8d4e096bef619e3541bb500744a6644abbd $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/radpaths.h>
@@ -176,9 +176,9 @@ typedef struct main_config {
 
 	bool		exiting;			//!< are we exiting?
 
-	bool		require_ma;			//!< global configuration for all clients and home servers
+	fr_bool_auto_t 	require_ma;			//!< global configuration for all clients and home servers
 
-	bool		limit_proxy_state;     		//!< global configuration for all clients
+	fr_bool_auto_t 	limit_proxy_state;     		//!< global configuration for all clients
 
 #ifdef ENABLE_OPENSSL_VERSION_CHECK
 	char const	*allow_vulnerable_openssl;	//!< The CVE number of the last security issue acknowledged.
@@ -199,9 +199,8 @@ typedef struct main_config {
 typedef enum {
 	REQUEST_ACTIVE = 1,
 	REQUEST_STOP_PROCESSING,
-	REQUEST_COUNTED
 } rad_master_state_t;
-#define REQUEST_MASTER_NUM_STATES (REQUEST_COUNTED + 1)
+#define REQUEST_MASTER_NUM_STATES (REQUEST_STOP_PROCESSING + 1)
 
 typedef enum {
 	REQUEST_QUEUED = 1,
@@ -325,6 +324,7 @@ struct rad_request {
 #define RAD_REQUEST_OPTION_COA		(1 << 0)
 #define RAD_REQUEST_OPTION_CTX 		(1 << 1)
 #define RAD_REQUEST_OPTION_CANCELLED	(1 << 2)
+#define RAD_REQUEST_OPTION_STATS	(1 << 3)
 
 #define SECONDS_PER_DAY		86400
 #define MAX_REQUEST_TIME	30
@@ -373,7 +373,8 @@ typedef enum {
 	RADIUS_SIGNAL_SELF_EXIT		= (1 << 2),
 	RADIUS_SIGNAL_SELF_DETAIL	= (1 << 3),
 	RADIUS_SIGNAL_SELF_NEW_FD	= (1 << 4),
-	RADIUS_SIGNAL_SELF_MAX		= (1 << 5)
+	RADIUS_SIGNAL_SELF_EVENT_UPDATE	= (1 << 5),
+	RADIUS_SIGNAL_SELF_MAX		= (1 << 6)
 } radius_signal_t;
 /*
  *	Function prototypes.
@@ -570,6 +571,8 @@ int main_config_free(void);
 void main_config_hup(void);
 void hup_logfile(void);
 
+int	fr_bool_auto_parse(CONF_PAIR *cp, fr_bool_auto_t *out, char const *str);
+
 /* listen.c */
 void listen_free(rad_listen_t **head);
 int listen_init(CONF_SECTION *cs, rad_listen_t **head, bool spawn_flag);
