@@ -1,7 +1,7 @@
 /*
  * stats.c	Internal statistics handling.
  *
- * Version:	$Id: 2f5298d384847a3eab9c928581b7fccb95e2dcbc $
+ * Version:	$Id: 64cbafea931f9e807cfa50984853e20b0e07173b $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2008  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: 2f5298d384847a3eab9c928581b7fccb95e2dcbc $")
+RCSID("$Id: 64cbafea931f9e807cfa50984853e20b0e07173b $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -91,51 +91,65 @@ static void stats_time(fr_stats_t *stats, struct timeval *start,
 void request_stats_final(REQUEST *request)
 {
 	rad_listen_t *listener;
+	RADCLIENT *client;
 
-	if (request->master_state == REQUEST_COUNTED) return;
+	if ((request->options & RAD_REQUEST_OPTION_STATS) != 0) return;
 
-	if (!request->listener) return;
-	if (!request->client) return;
-
-	if ((request->listener->type != RAD_LISTEN_NONE) &&
-#ifdef WITH_ACCOUNTING
-	    (request->listener->type != RAD_LISTEN_ACCT) &&
-#endif
-#ifdef WITH_COA
-	    (request->listener->type != RAD_LISTEN_COA) &&
-#endif
-	    (request->listener->type != RAD_LISTEN_AUTH)) return;
+	/*
+	 *	This packet was originated by the server, and not
+	 *	received from a client.  It's a status-server or home
+	 *	server "ping" packet.  So we ignore it for statistics
+	 *	purposes.
+	 */
+	if (!request->packet) return;
 
 	/* don't count statistic requests */
-	if (request->packet->code == PW_CODE_STATUS_SERVER)
+	if (request->packet->code == PW_CODE_STATUS_SERVER) {
 		return;
+	}
+
+	listener = request->listener;
+	if (listener) switch (listener->type) {
+		case RAD_LISTEN_NONE:
+#ifdef WITH_ACCOUNTING
+		case RAD_LISTEN_ACCT:
+#endif
+#ifdef WITH_COA
+		case RAD_LISTEN_COA:
+#endif
+		case RAD_LISTEN_AUTH:
+			break;
+
+		default:
+			return;
+	}
 
 	/*
 	 *	Deal with TCP / TLS issues.  The statistics are kept in the parent socket.
 	 */
-	listener = request->listener;
-	if (listener->parent) listener = listener->parent;
+	if (listener && listener->parent) listener = listener->parent;
+	client = request->client;
 
 #undef INC_AUTH
-#define INC_AUTH(_x) radius_auth_stats._x++;listener->stats._x++;request->client->auth._x++;
+#define INC_AUTH(_x) radius_auth_stats._x++;if (listener) listener->stats._x++;if (client) client->auth._x++;
 
 #undef INC_ACCT
 #ifdef WITH_ACCOUNTING
-#define INC_ACCT(_x) radius_acct_stats._x++;listener->stats._x++;request->client->acct._x++
+#define INC_ACCT(_x) radius_acct_stats._x++;if (listener) listener->stats._x++;if (client) client->acct._x++
 #else
 #define INC_ACCT(_x)
 #endif
 
 #undef INC_COA
 #ifdef WITH_COA
-#define INC_COA(_x) radius_coa_stats._x++;listener->stats._x++;request->client->coa._x++
+#define INC_COA(_x) radius_coa_stats._x++;if (listener) listener->stats._x++;if (client) client->coa._x++
 #else
 #define INC_COA(_x)
 #endif
 
 #undef INC_DSC
 #ifdef WITH_DSC
-#define INC_DSC(_x) radius_dsc_stats._x++;listener->stats._x++;request->client->dsc._x++
+#define INC_DSC(_x) radius_dsc_stats._x++;if (listener) listener->stats._x++;if (client) client->dsc._x++
 #else
 #define INC_DSC(_x)
 #endif
@@ -148,7 +162,7 @@ void request_stats_final(REQUEST *request)
 	 *	deleted, because only the main server thread calls
 	 *	this function, which makes it thread-safe.
 	 */
-	if (request->reply && (request->packet->code != PW_CODE_STATUS_SERVER)) switch (request->reply->code) {
+	if (request->reply) switch (request->reply->code) {
 	case PW_CODE_ACCESS_ACCEPT:
 		INC_AUTH(total_access_accepts);
 
@@ -247,25 +261,21 @@ void request_stats_final(REQUEST *request)
 	switch (request->proxy->code) {
 	case PW_CODE_ACCESS_REQUEST:
 		proxy_auth_stats.total_requests += request->num_proxied_requests;
-		request->home_server->stats.total_requests += request->num_proxied_requests;
 		break;
 
 #ifdef WITH_ACCOUNTING
 	case PW_CODE_ACCOUNTING_REQUEST:
 		proxy_acct_stats.total_requests += request->num_proxied_requests;
-		request->home_server->stats.total_requests += request->num_proxied_requests;
 		break;
 #endif
 
 #ifdef WITH_COA
 	case PW_CODE_COA_REQUEST:
 		proxy_coa_stats.total_requests += request->num_proxied_requests;
-		request->home_server->stats.total_requests += request->num_proxied_requests;
 		break;
 
 	case PW_CODE_DISCONNECT_REQUEST:
 		proxy_dsc_stats.total_requests += request->num_proxied_requests;
-		request->home_server->stats.total_requests += request->num_proxied_requests;
 		break;
 #endif
 
@@ -276,7 +286,7 @@ void request_stats_final(REQUEST *request)
 	if (!request->proxy_reply) goto done;	/* simplifies formatting */
 
 #undef INC
-#define INC(_x) proxy_auth_stats._x += request->num_proxied_responses; request->home_server->stats._x += request->num_proxied_responses;
+#define INC(_x) proxy_auth_stats._x += request->num_proxied_responses;request->home_server->stats._x += request->num_proxied_responses;
 
 	switch (request->proxy_reply->code) {
 	case PW_CODE_ACCESS_ACCEPT:
@@ -347,10 +357,7 @@ void request_stats_final(REQUEST *request)
  done:
 #endif /* WITH_PROXY */
 
-
 	if (request->max_time) {
-		RADCLIENT *client = request->client;
-
 		switch (request->packet->code) {
 		case PW_CODE_ACCESS_REQUEST:
 			FR_STATS_INC(auth, unresponsive_child);
@@ -376,7 +383,7 @@ void request_stats_final(REQUEST *request)
 		}
 	}
 
-	request->master_state = REQUEST_COUNTED;
+	request->options |= RAD_REQUEST_OPTION_STATS;
 }
 
 typedef struct fr_stats2vp {
